export const baseUrl = import.meta.env.VITE_BASE_URL as string;
export const token = () => localStorage.getItem("access_token") as string;

export default async function api(path: string, data: any): Promise<any> {
  const request = new Request(`${baseUrl}${path}/`, {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
    },
    body: JSON.stringify(data),
  });
  let response: Response;
  try {
    response = await fetch(request);
  } catch (e) {
    throw new Error(`request failed: ${e}`);
  }
  return response;
}

export async function apiAuth(
  path: string,
  data: any,
  method: string = "GET"
): Promise<any> {
  const req = new Request(`${baseUrl}api/${path}`, {
    method: method,
    headers: {
      Authorization: `Bearer ${token()} `,
      "Content-Type": "application/json",
    },
    ...(data && { body: JSON.stringify(data) }),
  });
  let resp: Response;
  try {
    resp = await fetch(req);
  } catch (e) {
    throw new Error(`request failed: ${e}`);
  }

  if (!resp.ok) {
    if (resp.status === 401) {
      logout();
      throw new Error("Unauthorized");
    }
  }
  return resp.json();
}

export type User = {
  username: string;
  full_name: string;
  email: string;
  player_id: number;
};

export async function currentUser(): Promise<User> {
  if (!token()) throw new Error("you have no access token");
  const req = new Request(`${baseUrl}api/users/me/`, {
    method: "GET",
    headers: {
      Authorization: `Bearer ${token()} `,
      "Content-Type": "application/json",
    },
  });
  let resp: Response;
  try {
    resp = await fetch(req);
  } catch (e) {
    throw new Error(`request failed: ${e}`);
  }

  if (!resp.ok) {
    if (resp.status === 401) {
      logout();
      throw new Error("Unauthorized");
    }
  }
  return resp.json() as Promise<User>;
}

export type LoginRequest = {
  username: string;
  password: string;
};
export type Token = {
  access_token: string;
  token_type: string;
};

// api.js
export const login = async (req: LoginRequest): Promise<void> => {
  try {
    const response = await fetch(`${baseUrl}api/token`, {
      method: "POST",
      headers: {
        "Content-Type": "application/x-www-form-urlencoded",
      },
      body: new URLSearchParams(req).toString(),
    });
    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }
    const token = (await response.json()) as Token;
    if (token && token.access_token) {
      localStorage.setItem("access_token", token.access_token);
    } else {
      console.log("Token not acquired");
    }
  } catch (e) {
    console.error(e);
    throw e; // rethrow the error so it can be caught by the caller
  }
};

export const logout = () => localStorage.removeItem("access_token");
